/*
 * Copyright (C) 2006-2007	Andre Beckedorf
 * 					 		<evilJazz _AT_ katastrophos _DOT_ net>
 *
 * This file is based on eMotion by Michael Swieton <mike@swieton.net>.
 * The few remaining parts in this almost completely rewritten file are
 * Copyright (C) 2005 Atmark <atmarkat _AT_ msn _DOT_ com>
 *                    AGAWA Koji <i _AT_ atty _DOT_ jp>
 *                    Michael Swieton <mike@swieton.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef MPLAYER_H_INCLUDED
#define MPLAYER_H_INCLUDED

#include <qobject.h>
#include <qstring.h>
#include <qtimer.h>
#include <qsocketnotifier.h>
#include <qtextstream.h>
#include <qprocess.h>

#include <sys/types.h>

#include "playbackbackend.h"
#include "mplayerconfig.h"
#include "system_volume.h"
#include "signalhandler.h"

class Media;

class MPlayerBackend: public AbstractPlaybackBackend, SignalHandler
{
	Q_OBJECT

public:
	MPlayerBackend(MPlayerConfiguration &config);
	~MPlayerBackend();

#ifdef QT4
	virtual bool isRunning() const { return process_ && process_->state() != QProcess::NotRunning; };
#else
	virtual bool isRunning() const { return process_ && process_->isRunning(); };
#endif
	virtual bool isPlaying() const { return isRunning() && playbackWasStarted_; }
	virtual bool isPaused() const { return isPaused_; };

signals:
	void killed();

public slots:
	virtual AbstractPlaybackBackend::PlayResult play(Media *);
	virtual void pause();
	virtual void stop() { stop(false); }
	virtual void seekRelativeSeconds(int);
	virtual void seekAbsolutePercentage(int);
	virtual void seekAbsoluteSeconds(int);

	void toggleOSD();
	void osdShowProgressBar(int);
	void enableTemporaryOSD();
	void disableTemporaryOSD();
	void setBrightness(int);

private slots:
	void cleanUp();
	void processReadyReadStdout();
	void processExited();
	void parseLines();
	void pollMPlayer();
	void checkFeaturesProcessParseLines();
	void systemVolumeChanged(int);

	void killWaitingProcesses();
	void updatedValues();

	void osdTimerTimedUpdate();

private:
	bool checkFeatures();

	QStringList getArgListForMedia(Media *);
	bool canRecycleMplayer(Media *, QStringList &);
	void scheduleDestructionOfProcess(QProcess *);

	void sendCommand(const QString&); //!< slaveコマンド送信
	void sendMedia(Media *);
	void updatePosition();
	void sendCurrentVolume();
	bool useSoftVolume() { return qSystemVolume.useInternalVolume(); }

	void checkFeaturesRunProcess(const QString &program, const QStringList &args);
	void stop(bool emitQuitSignal);
	void quitMplayer(bool emitQuitSignal);

	void handleSignal(int);
	bool canRecycleMplayer(Media *);

	MPlayerConfiguration &config_;

	/* MPlayer制御 */
	enum QuitState {
		QuitNull,
		QuitUser,
		QuitEOF
	};
	QuitState quitState_;		//!< 終了方法
	bool isPaused_;				//!< ポーズ状態
	bool playbackWasStarted_;
	bool errorDetected_;
	bool isSendingPTS_;			// is mplayer sending PTS=<currentplaytime> ?
	bool parsing_;
	int watchdogCheck_;

	QProcess *process_;
	QString lastMplayerBinLocation_;
	QStringList currentProcessArgs_;
	QString buffer_;
	QTimer timer_;
	QTimer *osdTimer_;

	QString messages_;
	QString params_;

	QTimer killTimer_;
#ifdef QT4
	QList<QProcess *> processKillWaitList_;
#else
	QList<QProcess> processKillWaitList_;
#endif

	int newPlaytime_;

	bool isOSDVisible_;			//!< OSDの現在の表示状態
	bool isTemporaryOSDEnabled_; //!< 一時的にOSDを表示するモードになっているか
	bool savedOSDVisiblity_;	//!< 一時的にOSDを表示する前のOSD表示状態
	QTextStream logStream_;		//!< MPlayerの出力を保存するストリーム
};

#endif // MPLAYER_H_INCLUDED
